==> ec2init-0.19.2-py3-none-any.whl: ec2init/plans/multinode/create_service_api_key.py <==
import logging
from configparser import ConfigParser

from typing import Dict, cast, List, Optional
from ec2init.execution.procedure import Procedure
import xml.etree.ElementTree as ET

from ec2init.plans.plan_utils import HopsPlanUtils
from ec2init.utils.common import CommonUtils
from ec2init import Procedure
from ec2init.utils.constants import Url
from ec2init.utils.http import HttpUtils
from ec2init.utils.dns import DnsUtils
from ec2init.utils.cloud import get_cloud_provider
from ec2init.utils.random_generator import RandomGenerator
from ec2init.utils.database import databaseUtils, ReplaceVariables
from ec2init.utils.system import SystemUtils

logger = logging.getLogger(__name__)

class CreateServiceApiKey(Procedure):

    @staticmethod
    def should_run(plan_utils: HopsPlanUtils) -> bool:
        check_file = plan_utils.get_procedure_check_file(CreateServiceApiKey.__name__)
        return not plan_utils.check_file_exists(check_file)

    def __init__(self, name: str, config: ConfigParser, plan_utils: HopsPlanUtils, cloud_provider: str) -> None:
        self.name = name
        self.config = config
        self.plan_utils = plan_utils
        self.cloud_provider = cloud_provider

    def execute(self) -> List[Procedure]:
        DnsUtils.get_ip_from_name("hopsworks.glassfish.service.consul")
        return []
    
    def _done(self) -> None:
        check_file = self.plan_utils.get_procedure_check_file(CreateServiceApiKey.__name__)
        check_file.touch()

    def _get_api_key(self, requester: str, scopes: List[str]) -> str:
        username, password = CommonUtils._get_agent_credentials(self.config)
        if not username or not password:
            raise Exception("Agent username or password is not set")
        
        method, uri = Url.HOPSWORKS_LOGIN
        data = {
            'email': username,
            'password': password,
        }
        headers = {
            'Content-Type': 'application/x-www-form-urlencoded',
        }
        response: Dict[str, str] = HttpUtils.do_plain_call(self.config, uri, method, data=data, headers=headers, verify=False)
        response_headers: Dict[str, str] = cast(Dict[str, str], response['headers'])
        jwt = response_headers['Authorization']

        # Make request to generate an API key
        ip = get_cloud_provider(self.cloud_provider).get_private_ip(self.config)
        rand = RandomGenerator()
        api_key_name = f"{requester}_{ip}_{rand.random_string(6)}"
        api_key_params = {
            'name': api_key_name,
            'scope': ",".join(scopes)
        }
        headers = {
            'Authorization': jwt
        }
        method, uri = Url.NEW_HOPSWORKS_API_KEY_URL
        api_key_response: Dict[str, str] = HttpUtils.do_api_call(self.config, uri, method, params=api_key_params, headers=headers, verify=False)
        api_key = api_key_response['key']
        if api_key == "":
            raise Exception(f"API key {api_key_name} is empty, something went wrong")
        return api_key
    
    def rollback(self) -> None:
        pass

class WorkerCreateServiceApiKey(CreateServiceApiKey):
    SSL_SERVER_XML = "/srv/hops/hadoop/etc/hadoop/ssl-server.xml"

    def __init__(self, name: str, config: ConfigParser, plan_utils: HopsPlanUtils, cloud_provider: str, existing_key: Optional[str]) -> None:
        super().__init__(name, config, plan_utils, cloud_provider)
        self.existing_key = existing_key

    def execute(self) -> List[Procedure]:
        logger.info(f"{self.name} - Replacing API key in ssl-server.xml")
        super().execute()

        # Replace API key in ssl-server.xml
        if self.existing_key:
            api_key = self.existing_key
        else:
            api_key = self._get_api_key("hops_worker", ["AUTH"])
        self._replace_key_ssl_server(api_key)
        logger.info(f"{self.name} - Replaced API key in ssl-server.xml")
        self._done()
        return []
    
    def _replace_key_ssl_server(self, api_key: str) -> None:    
        tree = ET.parse(WorkerCreateServiceApiKey.SSL_SERVER_XML)
        for p in tree.iter("property"):
            if p[0].text == "hops.hopsworks-api-key":
                p[1].text = api_key
                break
        tree.write(WorkerCreateServiceApiKey.SSL_SERVER_XML)
    
class HeadCreateServiceApiKey(CreateServiceApiKey):
    def __init__(self, name: str, config: ConfigParser, plan_utils: HopsPlanUtils, cloud_provider: str) -> None:
        super().__init__(name, config, plan_utils, cloud_provider)

    def execute(self) -> List[Procedure]:
        logger.info(f"{self.name} - Replacing API key in database")
        super().execute()

        # Replace API key in database
        api_key = self._get_api_key("hops_head", ["AUTH"])
        self._replace_database(api_key)
        
        sslServer = WorkerCreateServiceApiKey(self.name, self.config, self.plan_utils, self.cloud_provider, api_key)
        logger.info(f"{self.name} - Replaced API key in database")
        return [sslServer]

    def _replace_database(self, api_key) -> None:
        databaseUtils.execute_variables_query(ReplaceVariables("int_service_api_key", api_key))

==> ec2init-0.20-py3-none-any.whl: ec2init/plans/multinode/create_service_api_key.py <==
import logging
from configparser import ConfigParser

from typing import Dict, cast, List, Optional, Tuple
from ec2init.execution.procedure import Procedure
import xml.etree.ElementTree as ET

from ec2init.plans.plan_utils import HopsPlanUtils
from ec2init.utils.common import CommonUtils
from ec2init import Procedure
from ec2init.utils.constants import Url
from ec2init.utils.http import HttpUtils
from ec2init.utils.dns import DnsUtils
from ec2init.utils.cloud import get_cloud_provider
from ec2init.utils.random_generator import RandomGenerator
from ec2init.utils.database import databaseUtils, ReplaceVariables
from ec2init.utils.system import SystemUtils

logger = logging.getLogger(__name__)

class CreateServiceApiKey(Procedure):

    @staticmethod
    def should_run(plan_utils: HopsPlanUtils) -> bool:
        check_file = plan_utils.get_procedure_check_file(CreateServiceApiKey.__name__)
        return not plan_utils.check_file_exists(check_file)

    def __init__(self, name: str, config: ConfigParser, plan_utils: HopsPlanUtils, cloud_provider: str) -> None:
        self.name = name
        self.config = config
        self.plan_utils = plan_utils
        self.cloud_provider = cloud_provider

    def execute(self) -> List[Procedure]:
        DnsUtils.get_ip_from_name("hopsworks.glassfish.service.consul")
        return []
    
    def _done(self) -> None:
        check_file = self.plan_utils.get_procedure_check_file(CreateServiceApiKey.__name__)
        check_file.touch()

    def _get_api_key(self, requester: str, scopes: List[str]) -> str:
        username, password = self.get_user_credentials()
        if not username or not password:
            raise Exception("API user username or password is not set")
        
        method, uri = Url.HOPSWORKS_LOGIN
        data = {
            'email': username,
            'password': password,
        }
        headers = {
            'Content-Type': 'application/x-www-form-urlencoded',
        }
        response: Dict[str, str] = HttpUtils.do_plain_call(self.config, uri, method, data=data, headers=headers, verify=False)
        response_headers: Dict[str, str] = cast(Dict[str, str], response['headers'])
        jwt = response_headers['Authorization']

        # Make request to generate an API key
        ip = get_cloud_provider(self.cloud_provider).get_private_ip(self.config)
        rand = RandomGenerator()
        api_key_name = f"{requester}_{ip}_{rand.random_string(6)}"
        api_key_params = {
            'name': api_key_name,
            'scope': scopes
        }
        headers = {
            'Authorization': jwt
        }
        method, uri = Url.NEW_HOPSWORKS_API_KEY_URL
        api_key_response: Dict[str, str] = HttpUtils.do_api_call(self.config, uri, method, params=api_key_params, headers=headers, verify=False)
        api_key = api_key_response['key']
        if api_key == "":
            raise Exception(f"API key {api_key_name} is empty, something went wrong")
        return api_key
    
    def get_user_credentials(self) -> Tuple[str, str]:
        return CommonUtils._get_agent_credentials(self.config)

    def rollback(self) -> None:
        pass

class WorkerCreateServiceApiKey(CreateServiceApiKey):
    SSL_SERVER_XML = "/srv/hops/hadoop/etc/hadoop/ssl-server.xml"

    def __init__(self, name: str, config: ConfigParser, plan_utils: HopsPlanUtils, cloud_provider: str, existing_key: Optional[str]) -> None:
        super().__init__(name, config, plan_utils, cloud_provider)
        self.existing_key = existing_key

    def execute(self) -> List[Procedure]:
        logger.info(f"{self.name} - Replacing API key in ssl-server.xml")
        super().execute()

        # Replace API key in ssl-server.xml
        if self.existing_key:
            api_key = self.existing_key
        else:
            api_key = self._get_api_key("hops_worker", ["AUTH"])
        self._replace_key_ssl_server(api_key)
        logger.info(f"{self.name} - Replaced API key in ssl-server.xml")
        self._done()
        return []
    
    def _replace_key_ssl_server(self, api_key: str) -> None:    
        tree = ET.parse(WorkerCreateServiceApiKey.SSL_SERVER_XML)
        for p in tree.iter("property"):
            if p[0].text == "hops.hopsworks-api-key":
                p[1].text = api_key
                break
        tree.write(WorkerCreateServiceApiKey.SSL_SERVER_XML)
    
class HeadCreateServiceApiKey(CreateServiceApiKey):
    def __init__(self, name: str, config: ConfigParser, plan_utils: HopsPlanUtils, cloud_provider: str) -> None:
        super().__init__(name, config, plan_utils, cloud_provider)

    def execute(self) -> List[Procedure]:
        logger.info(f"{self.name} - Replacing API key in database")
        super().execute()

        # Replace API key in database
        api_key = self._get_api_key("hops_head", ["AUTH"])
        self._replace_database(api_key)
        
        sslServer = WorkerCreateServiceApiKey(self.name, self.config, self.plan_utils, self.cloud_provider, api_key)
        logger.info(f"{self.name} - Replaced API key in database")
        return [sslServer]

    def _replace_database(self, api_key) -> None:
        databaseUtils.execute_variables_query(ReplaceVariables("int_service_api_key", api_key))
